@@ -71,7 +71,7 @@ from deezspot.models.callback.callbacks import (
 from deezspot.models.callback.track import trackObject as trackCbObject, albumTrackObject, artistTrackObject, playlistTrackObject
 from deezspot.models.callback.album import albumObject as albumCbObject
 from deezspot.models.callback.playlist import playlistObject as playlistCbObject
-from deezspot.models.callback.common import IDs
+from deezspot.models.callback.common import IDs, Service
 from deezspot.models.callback.user import userObject
 
 # Use unified metadata converter
@@ -381,6 +381,18 @@ class EASY_DW:
         custom_track_format = getattr(self.__preferences, 'custom_track_format', None)
         pad_tracks = getattr(self.__preferences, 'pad_tracks', True)
         self.__song_metadata_dict['artist_separator'] = getattr(self.__preferences, 'artist_separator', '; ')
+        # Determine pad width (supports 'auto' mode) based on context
+        pad_number_width = None
+        try:
+            pnw = getattr(self.__preferences, 'pad_number_width', None)
+            if isinstance(pnw, str) and pnw.lower() == 'auto':
+                total = getattr(self.__preferences, 'total_tracks', None)
+                if isinstance(total, int) and total and total > 0:
+                    pad_number_width = max(2, len(str(total)))
+            elif isinstance(pnw, int) and pnw >= 1:
+                pad_number_width = pnw
+        except Exception:
+            pad_number_width = None
         # Inject playlist placeholders if in playlist context
         try:
             if self.__parent == 'playlist' and hasattr(self.__preferences, 'json_data') and self.__preferences.json_data:
@@ -404,7 +416,8 @@ class EASY_DW:
             self.__file_format,
             custom_dir_format=custom_dir_format,
             custom_track_format=custom_track_format,
-            pad_tracks=pad_tracks
+            pad_tracks=pad_tracks,
+            pad_number_width=pad_number_width
         )
     
     def __set_episode_path(self) -> None:
@@ -548,9 +561,31 @@ class EASY_DW:
                         summary = summaryObject(
                             successful_tracks=[self.__track_obj],
                             total_successful=1,
+                            service=Service.DEEZER,
                         )
                         summary.final_path = final_path_val
                         summary.download_quality = download_quality_val
+                        # Compute final quality/bitrate
+                        quality_val = None
+                        bitrate_val = None
+                        if self.__convert_to:
+                            fmt, brp = self._parse_format_string(self.__convert_to)
+                            if fmt:
+                                quality_val = fmt.lower()
+                            if brp:
+                                bitrate_val = brp.lower().replace('kbps', 'k')
+                        else:
+                            qkey = (self.__quality_download or '').upper()
+                            if qkey.startswith('MP3'):
+                                quality_val = 'mp3'
+                                try:
+                                    bitrate_val = f"{qkey.split('_')[1]}k"
+                                except Exception:
+                                    bitrate_val = None
+                            elif qkey == 'FLAC':
+                                quality_val = 'flac'
+                        summary.quality = quality_val
+                        summary.bitrate = bitrate_val
                         done_status.summary = summary
                         
                     callback_obj = trackCallbackObject(
@@ -1260,6 +1295,8 @@ class DW_ALBUM:
                 c_preferences.track_number = a + 1  # For progress reporting only
                 c_preferences.total_tracks = total_tracks
                 c_preferences.link = f"https://deezer.com/track/{c_preferences.ids}"
+                # Propagate pad width preference
+                c_preferences.pad_number_width = getattr(self.__preferences, 'pad_number_width', 'auto')
                 # Inject Spotify per-track object if available without extra API calls
                 if self.__use_spotify and spotify_tracks_in_order:
                     spo_track_for_tag = None
@@ -1325,8 +1362,32 @@ class DW_ALBUM:
             failed_tracks=failed_tracks_cb,
             total_successful=len(successful_tracks_cb),
             total_skipped=len(skipped_tracks_cb),
-            total_failed=len(failed_tracks_cb)
+            total_failed=len(failed_tracks_cb),
+            service=Service.DEEZER
         )
+        # Compute and attach final media characteristics
+        quality_val = None
+        bitrate_val = None
+        if getattr(self.__preferences, 'convert_to', None):
+            fmt = getattr(self.__preferences, 'convert_to', None)
+            if fmt:
+                quality_val = fmt.lower()
+            br_raw = getattr(self.__preferences, 'bitrate', None)
+            if br_raw:
+                digits = ''.join([c for c in str(br_raw) if c.isdigit()])
+                bitrate_val = f"{digits}k" if digits else None
+        else:
+            qkey = (self.__quality_download or '').upper()
+            if qkey.startswith('MP3'):
+                quality_val = 'mp3'
+                try:
+                    bitrate_val = f"{qkey.split('_')[1]}k"
+                except Exception:
+                    bitrate_val = None
+            elif qkey == 'FLAC':
+                quality_val = 'flac'
+        summary_obj.quality = quality_val
+        summary_obj.bitrate = bitrate_val
         
         # Report album completion status
         report_album_done(album_obj, summary_obj)
@@ -1410,6 +1471,8 @@ class DW_PLAYLIST:
             c_preferences.total_tracks = total_tracks
             c_preferences.json_data = self.__preferences.json_data
             c_preferences.link = f"https://deezer.com/track/{c_preferences.ids}"
+            # Propagate pad width preference
+            c_preferences.pad_number_width = getattr(self.__preferences, 'pad_number_width', 'auto')
 
             current_track_object = None
             try:
@@ -1447,8 +1510,32 @@ class DW_PLAYLIST:
             failed_tracks=failed_tracks_cb,
             total_successful=len(successful_tracks_cb),
             total_skipped=len(skipped_tracks_cb),
-            total_failed=len(failed_tracks_cb)
+            total_failed=len(failed_tracks_cb),
+            service=Service.DEEZER
         )
+        # Compute and attach final media characteristics
+        quality_val = None
+        bitrate_val = None
+        if getattr(self.__preferences, 'convert_to', None):
+            fmt = getattr(self.__preferences, 'convert_to', None)
+            if fmt:
+                quality_val = fmt.lower()
+            br_raw = getattr(self.__preferences, 'bitrate', None)
+            if br_raw:
+                digits = ''.join([c for c in str(br_raw) if c.isdigit()])
+                bitrate_val = f"{digits}k" if digits else None
+        else:
+            qkey = (self.__quality_download or '').upper()
+            if qkey.startswith('MP3'):
+                quality_val = 'mp3'
+                try:
+                    bitrate_val = f"{qkey.split('_')[1]}k"
+                except Exception:
+                    bitrate_val = None
+            elif qkey == 'FLAC':
+                quality_val = 'flac'
+        summary_obj.quality = quality_val
+        summary_obj.bitrate = bitrate_val
         
         # Attach m3u path to summary
         summary_obj.m3u_path = m3u_path
