@@ -47,6 +47,7 @@ from deezspot.libutils.others_settings import (
 )
 from deezspot.libutils.logging_utils import ProgressReporter, logger, report_progress
 import requests
+from librespot.core import Session
 
 from deezspot.models.callback.callbacks import (
     trackCallbackObject,
@@ -90,6 +91,7 @@ class DeeLogin:
         password=None,
         spotify_client_id=None,
         spotify_client_secret=None,
+        spotify_credentials_path=None,
         progress_callback=None,
         silent=False
     ) -> None:
@@ -97,6 +99,12 @@ class DeeLogin:
         # Store Spotify credentials
         self.spotify_client_id = spotify_client_id
         self.spotify_client_secret = spotify_client_secret
+        # Optional path to Spotify credentials.json (explicit param > env override > CWD default)
+        self.spotify_credentials_path = (
+            spotify_credentials_path
+            or os.environ.get("SPOTIFY_CREDENTIALS_PATH")
+            or os.path.join(os.getcwd(), "credentials.json")
+        )
         
         # Initialize Spotify API if credentials are provided
         if spotify_client_id and spotify_client_secret:
@@ -120,6 +128,27 @@ class DeeLogin:
         # Set the progress reporter for Download_JOB
         Download_JOB.set_progress_reporter(self.progress_reporter)
 
+    def _ensure_spotify_session(self) -> None:
+        """Ensure Spo has an attached librespot Session. Used only by spo->dee flows."""
+        try:
+            # Check if Spo already has a session (accessing private attr is ok internally)
+            has_session = getattr(Spo, f"_{Spo.__name__}__session", None) is not None
+            if has_session:
+                return
+        except Exception:
+            pass
+
+        cred_path = self.spotify_credentials_path
+        if not os.path.isfile(cred_path):
+            raise FileNotFoundError(
+                f"Spotify session not initialized. Missing credentials.json at '{cred_path}'. "
+                "Set SPOTIFY_CREDENTIALS_PATH or place credentials.json in the working directory."
+            )
+        builder = Session.Builder()
+        builder.conf.stored_credentials_file = cred_path
+        session = builder.stored_file().create()
+        Spo.set_session(session)
+
     def download_trackdee(
         self, link_track,
         output_dir=stock_output,
@@ -139,7 +168,8 @@ class DeeLogin:
         market=stock_market,
         playlist_context=None,
         artist_separator: str = "; ",
-        spotify_metadata: bool = False
+        spotify_metadata: bool = False,
+        pad_number_width: int | str = 'auto'
     ) -> Track:
 
         link_is_valid(link_track)
@@ -215,6 +245,7 @@ class DeeLogin:
         preferences.artist_separator = artist_separator
         preferences.spotify_metadata = bool(spotify_metadata)
         preferences.spotify_track_obj = playlist_context.get('spotify_track_obj') if (playlist_context and playlist_context.get('spotify_track_obj')) else None
+        preferences.pad_number_width = pad_number_width
 
         if playlist_context:
             preferences.json_data = playlist_context.get('json_data')
@@ -252,7 +283,8 @@ class DeeLogin:
         playlist_context=None,
         artist_separator: str = "; ",
         spotify_metadata: bool = False,
-        spotify_album_obj=None
+        spotify_album_obj=None,
+        pad_number_width: int | str = 'auto'
     ) -> Album:
 
         link_is_valid(link_album)
@@ -302,6 +334,7 @@ class DeeLogin:
         preferences.artist_separator = artist_separator
         preferences.spotify_metadata = bool(spotify_metadata)
         preferences.spotify_album_obj = spotify_album_obj
+        preferences.pad_number_width = pad_number_width
 
         if playlist_context:
             preferences.json_data = playlist_context['json_data']
@@ -335,7 +368,8 @@ class DeeLogin:
         bitrate=None,
         save_cover=stock_save_cover,
         market=stock_market,
-        artist_separator: str = "; "
+        artist_separator: str = "; ",
+        pad_number_width: int | str = 'auto'
     ) -> Playlist:
 
         link_is_valid(link_playlist)
@@ -370,6 +404,7 @@ class DeeLogin:
         preferences.save_cover = save_cover
         preferences.market = market
         preferences.artist_separator = artist_separator
+        preferences.pad_number_width = pad_number_width
 
         playlist = DW_PLAYLIST(preferences).dw()
 
@@ -388,7 +423,8 @@ class DeeLogin:
         convert_to=None,
         bitrate=None,
         save_cover=stock_save_cover,
-        market=stock_market
+        market=stock_market,
+        pad_number_width: int | str = 'auto'
     ) -> list[Track]:
 
         link_is_valid(link_artist)
@@ -408,7 +444,8 @@ class DeeLogin:
                 convert_to=convert_to,
                 bitrate=bitrate,
                 save_cover=save_cover,
-                market=market
+                market=market,
+                pad_number_width=pad_number_width
             )
             for track in top_tracks_json
         ]
@@ -416,6 +453,8 @@ class DeeLogin:
         return names
 
     def convert_spoty_to_dee_link_track(self, link_track):
+        # Ensure Spotify session only when using spo->dee conversion
+        self._ensure_spotify_session()
         link_is_valid(link_track)
         ids = get_ids(link_track)
         
@@ -508,6 +547,8 @@ class DeeLogin:
         return track_link_dee
 
     def convert_spoty_to_dee_link_album(self, link_album):
+        # Ensure Spotify session only when using spo->dee conversion
+        self._ensure_spotify_session()
         link_is_valid(link_album)
         ids = get_ids(link_album)
         
@@ -519,7 +560,7 @@ class DeeLogin:
         spo_artists = spotify_album_data.get('artists') or []
         spo_main_artist = (spo_artists[0].get('name') if spo_artists else '') or ''
         external_ids = spotify_album_data.get('external_ids') or {}
-        spo_upc = str(external_ids.get('upc') or '').strip()
+        spo_upc = str(external_ids.get('upc') or '').strip().lstrip('0')
         
         # Try UPC first
         if spo_upc:
@@ -547,7 +588,7 @@ class DeeLogin:
                 dzc = API.get_album_json(str(c_id))
             except Exception:
                 continue
-            upc = str(dzc.get('upc') or '').strip()
+            upc = str(dzc.get('upc') or '').strip().lstrip('0')
             if spo_upc and upc and spo_upc != upc:
                 continue
             link_dee = f"https://www.deezer.com/album/{c_id}"
@@ -574,7 +615,8 @@ class DeeLogin:
         market=stock_market,
         playlist_context=None,
         artist_separator: str = "; ",
-        spotify_metadata: bool = False
+        spotify_metadata: bool = False,
+        pad_number_width: int | str = 'auto'
     ) -> Track:
 
         link_dee = self.convert_spoty_to_dee_link_track(link_track)
@@ -612,7 +654,8 @@ class DeeLogin:
             market=market,
             playlist_context=playlist_context,
             artist_separator=artist_separator,
-            spotify_metadata=spotify_metadata
+            spotify_metadata=spotify_metadata,
+            pad_number_width=pad_number_width
         )
 
         return track
@@ -637,13 +680,17 @@ class DeeLogin:
         market=stock_market,
         playlist_context=None,
         artist_separator: str = "; ",
-        spotify_metadata: bool = False
+        spotify_metadata: bool = False,
+        spotify_album_obj=None,
+        pad_number_width: int | str = 'auto'
     ) -> Album:
 
         link_dee = self.convert_spoty_to_dee_link_album(link_album)
 
         spotify_album_obj = None
         if spotify_metadata:
+            # Only initialize Spotify session when we actually need Spotify metadata
+            self._ensure_spotify_session()
             try:
                 # Fetch full Spotify album with tracks once and convert to albumObject
                 from deezspot.spotloader.__spo_api__ import tracking_album as spo_tracking_album
@@ -672,7 +719,8 @@ class DeeLogin:
             playlist_context=playlist_context,
             artist_separator=artist_separator,
             spotify_metadata=spotify_metadata,
-            spotify_album_obj=spotify_album_obj
+            spotify_album_obj=spotify_album_obj,
+            pad_number_width=pad_number_width
         )
 
         return album
@@ -696,13 +744,41 @@ class DeeLogin:
         save_cover=stock_save_cover,
         market=stock_market,
         artist_separator: str = "; ",
-        spotify_metadata: bool = False
+        spotify_metadata: bool = False,
+        pad_number_width: int | str = 'auto'
     ) -> Playlist:
 
         link_is_valid(link_playlist)
         ids = get_ids(link_playlist)
 
+        # Ensure Spotify session for fetching playlist and tracks
+        self._ensure_spotify_session()
+
         playlist_json = Spo.get_playlist(ids)
+        # Ensure we keep the playlist ID for callbacks
+        if 'id' not in playlist_json:
+            playlist_json['id'] = ids
+        
+        # Enrich items with full track objects so downstream expects Web API shape
+        try:
+            items = playlist_json.get('tracks', {}).get('items', []) or []
+            track_ids = [it.get('track', {}).get('id') for it in items if it.get('track') and it['track'].get('id')]
+            full = Spo.get_tracks(track_ids) if track_ids else {'tracks': []}
+            full_list = full.get('tracks') or []
+            full_by_id = {t.get('id'): t for t in full_list if t and t.get('id')}
+            new_items = []
+            for it in items:
+                tid = (it.get('track') or {}).get('id')
+                full_track = full_by_id.get(tid)
+                if full_track:
+                    new_items.append({'track': full_track})
+                else:
+                    new_items.append(it)
+            playlist_json['tracks']['items'] = new_items
+        except Exception:
+            # If enrichment fails, continue with minimal ids
+            pass
+
         
         # Extract track metadata for playlist callback object
         playlist_tracks_for_callback = []
@@ -779,7 +855,7 @@ class DeeLogin:
         playlist_obj = playlistCbObject(
             title=playlist_json['name'],
             owner=userObject(name=playlist_json.get('owner', {}).get('display_name', 'Unknown Owner')),
-            ids=IDs(spotify=playlist_json['id']),
+            ids=IDs(spotify=playlist_json.get('id', ids)),
             tracks=playlist_tracks_for_callback  # Populate tracks array with track objects
         )
 
@@ -819,6 +895,10 @@ class DeeLogin:
             track_name = track_info.get('name', 'Unknown Track')
             artist_name = track_info['artists'][0]['name'] if track_info.get('artists') else 'Unknown Artist'
             link_track = track_info.get('external_urls', {}).get('spotify')
+            if not link_track:
+                tid = track_info.get('id')
+                if tid:
+                    link_track = f"https://open.spotify.com/track/{tid}"
 
             if not link_track:
                 logger.warning(f"The track \"{track_name}\" is not available on Spotify :(")
@@ -832,12 +912,8 @@ class DeeLogin:
                     'spotify_url': link_track
                 }
                 # Attach Spotify track object for tagging if requested
-                if spotify_metadata:
-                    try:
-                        from deezspot.spotloader.__spo_api__ import json_to_track_playlist_object
-                        playlist_ctx['spotify_track_obj'] = json_to_track_playlist_object(track_info)
-                    except Exception:
-                        pass
+                if False: # placeholder, will be handled via spotify_metadata in download_trackspo
+                    pass
                 downloaded_track = self.download_trackspo(
                     link_track,
                     output_dir=output_dir, quality_download=quality_download,
@@ -847,7 +923,8 @@ class DeeLogin:
                     initial_retry_delay=initial_retry_delay, retry_delay_increase=retry_delay_increase,
                     max_retries=max_retries, convert_to=convert_to, bitrate=bitrate,
                     save_cover=save_cover, market=market, playlist_context=playlist_ctx,
-                    artist_separator=artist_separator, spotify_metadata=spotify_metadata
+                    artist_separator=artist_separator, spotify_metadata=False,
+                    pad_number_width=pad_number_width
                 )
                 tracks.append(downloaded_track)
                 
@@ -918,7 +995,8 @@ class DeeLogin:
         bitrate=None,
         save_cover=stock_save_cover,
         market=stock_market,
-        artist_separator: str = "; "
+        artist_separator: str = "; ",
+        pad_number_width: int | str = 'auto'
     ) -> Track:
 
         query = f"track:{song} artist:{artist}"
@@ -949,7 +1027,8 @@ class DeeLogin:
             bitrate=bitrate,
             save_cover=save_cover,
             market=market,
-            artist_separator=artist_separator
+            artist_separator=artist_separator,
+            pad_number_width=pad_number_width
         )
 
         return track
