@@ -3,7 +3,7 @@
 from dataclasses import dataclass, field
 from typing import List, Optional, Dict, Any, Union
 
-from .common import IDs
+from .common import IDs, Service
 from .track import trackObject, albumTrackObject, playlistTrackObject
 from .album import albumObject
 from .playlist import playlistObject
@@ -70,10 +70,14 @@ class summaryObject:
     total_successful: int = 0
     total_skipped: int = 0
     total_failed: int = 0
+    service: Optional[Service] = None
     # Extended info
     m3u_path: Optional[str] = None
     final_path: Optional[str] = None
     download_quality: Optional[str] = None
+    # Final media characteristics
+    quality: Optional[str] = None   # e.g., "mp3", "flac", "ogg"
+    bitrate: Optional[str] = None   # e.g., "320k"
 
 
 @dataclass
